
# q-expansion of newform 2700.2.a.h, downloaded from the LMFDB on 27 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2700
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [0], [-2], [2], [6], [1], [-3], [7], [-6], [1], [-8], [-10], [-2], [8], [9], [10], [5], [8], [6], [-2], [15], [13], [2], [8], [16], [-16], [12], [-11], [18], [16], [0], [3], [4], [10], [4], [-10], [-4], [15], [-1], [-20], [5], [-24], [-2], [-13], [-24], [3], [4], [1], [-13], [2], [-28], [7], [12], [-9], [16], [24], [-11], [-26], [6], [26], [-33], [-12], [18], [-20], [-11], [28], [22], [24], [25], [2], [12], [-2], [12], [-25], [-25], [-10], [22], [28], [5], [2], [-5], [-22], [-38], [33], [-15], [-6], [28], [-24], [-24], [21], [2], [-18], [-12], [9], [-21], [-24], [-2], [-22], [22], [22], [-18], [-4], [14], [37], [22], [15], [-9], [6], [5], [14], [16], [1], [-4], [-37], [32], [-6], [-21], [-23], [-38], [-10], [6], [-38], [-23], [-17], [-28], [-6], [-16], [-12], [8], [-29], [-12], [11], [-20], [-18], [-15], [-39], [10], [-41], [6], [-8], [4], [54], [7], [6], [4], [-4], [-3], [23], [39], [-46], [30], [-42], [27], [-24], [54], [0], [6], [-28], [20], [-3], [38], [20], [24], [-6], [-37], [-17], [-30], [11], [39], [-24], [-57], [2], [32], [31], [-42], [-43], [30], [-2], [-22], [34], [28], [-40], [42], [29], [0], [34], [-52], [-31], [-38], [0], [43], [-65], [-48], [33], [15], [9], [36], [30], [56], [-18], [-24], [48], [-17], [-34], [-30], [48], [-11], [16], [4], [-58], [18], [8], [-12], [-32], [-35], [-10], [-32], [11], [-35], [6], [-59], [-18], [-22], [-63], [-21], [67], [70], [-16], [-40], [2], [1], [-16], [60], [34], [13], [45], [-61], [0], [-18], [12], [41], [64], [1], [58], [12], [-12], [6], [20], [1], [12], [32], [17], [-71], [29], [-48], [61], [-70], [5], [-22], [18], [7], [7], [-16], [15], [-28], [0], [-52], [52], [-18], [-73], [-40], [-10], [-11], [46], [-22], [35], [-43], [41], [-30], [-24], [-31], [-61], [-65], [32], [-46], [12], [-58], [-25], [-16], [-16], [-81], [66], [12], [-62], [-36], [-3], [-54], [-40], [42], [14], [45], [32], [-3], [83], [34], [20], [35], [-50], [-54], [33], [36], [60], [-66], [-83], [-1], [40], [-2], [-86], [60], [-13], [-4], [36], [-68], [45], [62], [27], [60], [-20], [19], [46], [46], [-4], [-76], [-60], [36], [55], [21], [-74], [70], [-36], [53], [74], [-61], [-62], [78], [39], [92], [28], [27], [-88], [36], [-30], [60], [14], [-51], [90], [-42], [46], [-65], [-62], [-96], [24], [44], [-12], [94], [-99], [-36], [6], [52], [-43], [1], [-66], [69], [-12], [-40], [40], [66], [48], [-28], [60], [-1], [-46], [65], [84], [24], [75], [62], [30], [52], [-22], [-99], [-48], [-28], [-44], [58], [-1], [-62], [11], [-12], [-98], [-2], [74], [6], [-66], [-7], [24], [96], [-4], [30], [-40], [87], [-21], [-83], [-44], [-46], [-76], [0], [7], [-39], [60], [-28], [-23], [12], [-76], [-45], [71], [52], [-21], [-70]]
